-- Database framework already created
-- This script creates the tables - using GUIDs as keys

USE SalesDBGUIDs
go

select newid() AS CustomerGUID, * 
into SalesDBGUIDs.dbo.Customers
from Customers

select * from Customers

select newid() AS EmployeeGUID, * 
into SalesDBGUIDs.dbo.Employees
from SalesDB.dbo.Employees

select * from SalesDBGUIDs.dbo.Employees

select newid() AS ProductGUID, * 
into SalesDBGUIDs.dbo.Products
from SalesDB.dbo.Products

select * from SalesDBGUIDs.dbo.Products

select newid() AS SalesGUID, * 
into SalesDBGUIDs.dbo.Sales
from SalesDB.dbo.Sales

select NEWID() AS SalesGUID, 
	e.EmployeeGUID AS SalesPersonGUID,
	c.CustomerGUID AS CustomerGUID,
	p.ProductGUID AS ProductGUID,
	s.Quantity AS Quantity
into SalesDBGUIDs.dbo.Sales
from SalesDB.dbo.Sales as s
	join Products as p 
		on s.ProductID = p.ProductID
	join Customers as c
		on s.CustomerID = c.CustomerID
	join Employees as e
		on s.SalesPersonID = e.EmployeeID

select top 100 * from SalesDBGUIDs.dbo.Sales
select top 100 * from SalesDB.dbo.Sales

-- remove the old IDs

ALTER TABLE Products
	DROP COLUMN ProductID
	
ALTER TABLE Customers
	DROP COLUMN CustomerID
	
ALTER TABLE Employees
	DROP COLUMN EmployeeID
go

sp_help Products
exec sp_help Customers
exec sp_help Employees
exec sp_help Sales
go

-- Add the GUIDs as default newid() (same effect as client-side generation)

ALTER TABLE SalesDBGUIDs.dbo.Sales
ADD CONSTRAINT DF_Sales_RandomGUIDs 
		DEFAULT newid() FOR SalesGUID
go