-- Database framework already created
-- This script creates the tables - using GUIDs as keys

USE SalesDBSeqGUIDs
go

---------------------------------------------------------------------
-- Setting up Customers with sequential GUIDs
---------------------------------------------------------------------

SELECT convert(uniqueidentifier, '4D19C8BA-8654-4B42-B657-438E9AA7F5D1') AS CustomerGUID, * 
INTO SalesDBSeqGUIDs.dbo.Customers
FROM SalesDB.dbo.Customers AS c
ORDER BY c.CustomerID

UPDATE SalesDBSeqGUIDs.dbo.Customers
	SET CustomerGUID = NULL
go

ALTER TABLE SalesDBSeqGUIDs.dbo.Customers
ADD CONSTRAINT DF_Customers_SequentialGUIDs 
		DEFAULT newsequentialid() FOR CustomerGUID
go

UPDATE SalesDBSeqGUIDs.dbo.Customers
	SET CustomerGUID = DEFAULT
go
	
ALTER TABLE SalesDBSeqGUIDs.dbo.Customers
ALTER COLUMN
	CustomerGUID uniqueidentifier NOT NULL
go

SELECT * FROM SalesDBSeqGUIDs.dbo.Customers

---------------------------------------------------------------------
-- Setting up Employees with sequential GUIDs
---------------------------------------------------------------------

SELECT convert(uniqueidentifier, '4D19C8BA-8654-4B42-B657-438E9AA7F5D1') AS EmployeeGUID, * 
INTO SalesDBSeqGUIDs.dbo.Employees
FROM SalesDB.dbo.Employees AS e
ORDER BY e.EmployeeID

UPDATE SalesDBSeqGUIDs.dbo.Employees
	SET EmployeeGUID = NULL
go

ALTER TABLE SalesDBSeqGUIDs.dbo.Employees
ADD CONSTRAINT DF_Employees_SequentialGUIDs 
		DEFAULT newsequentialid() FOR EmployeeGUID
go

UPDATE SalesDBSeqGUIDs.dbo.Employees
	SET EmployeeGUID = DEFAULT
go
	
ALTER TABLE SalesDBSeqGUIDs.dbo.Employees
ALTER COLUMN
	EmployeeGUID uniqueidentifier NOT NULL
go

SELECT * FROM SalesDBSeqGUIDs.dbo.Employees

---------------------------------------------------------------------
-- Setting up Employees with sequential GUIDs
---------------------------------------------------------------------

SELECT convert(uniqueidentifier, '4D19C8BA-8654-4B42-B657-438E9AA7F5D1') AS ProductGUID, * 
INTO SalesDBSeqGUIDs.dbo.Products
FROM SalesDB.dbo.Products as p
ORDER BY p.ProductID

UPDATE SalesDBSeqGUIDs.dbo.Products
	SET ProductGUID = NULL
go

ALTER TABLE SalesDBSeqGUIDs.dbo.Products
ADD CONSTRAINT DF_Products_SequentialGUIDs 
		DEFAULT newsequentialid() FOR ProductGUID
go

UPDATE SalesDBSeqGUIDs.dbo.Products
	SET ProductGUID = DEFAULT
go
	
ALTER TABLE SalesDBSeqGUIDs.dbo.Products
ALTER COLUMN
	ProductGUID uniqueidentifier NOT NULL
go

SELECT * FROM SalesDBSeqGUIDs.dbo.Products
go

---------------------------------------------------------------------
-- Setting up Sales with sequential GUIDs
---------------------------------------------------------------------

SELECT convert(uniqueidentifier, '4D19C8BA-8654-4B42-B657-438E9AA7F5D1') AS SalesGUID, 
	e.EmployeeGUID AS SalesPersonGUID,
	c.CustomerGUID AS CustomerGUID,
	p.ProductGUID AS ProductGUID,
	s.Quantity AS Quantity
INTO SalesDBSeqGUIDs.dbo.Sales
FROM SalesDB.dbo.Sales AS s
	JOIN Products AS p 
		ON s.ProductID = p.ProductID
	JOIN Customers AS c
		ON s.CustomerID = c.CustomerID
	JOIN Employees AS e
		ON s.SalesPersonID = e.EmployeeID
ORDER BY s.SalesID
go

UPDATE SalesDBSeqGUIDs.dbo.Sales
	SET SalesGUID = NULL
go

ALTER TABLE SalesDBSeqGUIDs.dbo.Sales
ADD CONSTRAINT DF_Sales_SequentialGUIDs 
		DEFAULT newsequentialid() FOR SalesGUID
go

UPDATE SalesDBSeqGUIDs.dbo.Sales
	SET SalesGUID = DEFAULT
go
	
ALTER TABLE SalesDBSeqGUIDs.dbo.Sales
ALTER COLUMN
	SalesGUID uniqueidentifier NOT NULL
go

SELECT * FROM SalesDBSeqGUIDs.dbo.Sales
go

select top 100 * from SalesDBSeqGUIDs.dbo.Sales
select top 100 * from SalesDB.dbo.Sales

-- remove the old IDs

ALTER TABLE Products
	DROP COLUMN ProductID
	
ALTER TABLE Customers
	DROP COLUMN CustomerID
	
ALTER TABLE Employees
	DROP COLUMN EmployeeID
go

sp_help Products
exec sp_help Customers
exec sp_help Employees
exec sp_help Sales
go

-- Add the GUIDs as default newsequentialid() (same effect as client-side generation)
